#include <stdio.h>
#include "../lamport.h"
#include "../randombytes.h"

#define MAXMLEN 100

int main(void)
{
  unsigned char sk[CRYPTO_OTS_SECRETKEYBYTES];
  unsigned char pk[CRYPTO_OTS_PUBLICKEYBYTES];
  unsigned char sig[CRYPTO_OTS_SIGBYTES];
  unsigned char m[MAXMLEN];
  unsigned long long mlen;
  unsigned int pos;

  for(mlen=0;mlen<MAXMLEN;mlen++)
  {
    randombytes(m,mlen);
    crypto_ots_keypair(pk, sk);
    crypto_ots_sign(sig,m,mlen,sk);
    if(crypto_ots_verify(m,mlen,sig,pk))
      printf("Error: Failure verifying valid signature\n");

    /* check with modified signature */
    randombytes((unsigned char *)&pos, sizeof(int));
    pos %= CRYPTO_OTS_SIGBYTES;
    sig[pos] ^= 42;
    if(!crypto_ots_verify(m,mlen,sig,pk))
      printf("Error: Invalid signatures passes verification\n");

    sig[pos] ^= 42; /*restore original signature */
    /* check with modified message */
    if(mlen > 0)
    {
      randombytes((unsigned char *)&pos, sizeof(int));
      pos %= mlen;
      m[pos] ^= 42;
      if(!crypto_ots_verify(m,mlen,sig,pk))
        printf("Error: Invalid message passes verification\n");
    }

  }
  return 0;
}
